@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2009 by The BRLTTY Developers.
+ * Copyright (C) 1995-2010 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -23,7 +23,10 @@
 
 #include "prologue.h"
 
-#include "misc.h"
+#include <stdio.h>
+
+#include "log.h"
+#include "timing.h"
 #include "message.h"
 #include "eu_protocol.h"
 #include "eu_keys.h"
@@ -340,7 +343,7 @@ int	esysiris_init(BrailleDisplay *brl, t_eubrl_io *io)
       int i=60;
       while(i-- && brlCols==0)
         {
-          esysiris_readCommand(brl, BRL_CTX_SCREEN);
+          esysiris_readCommand(brl, BRL_CTX_DEFAULT);
           approximateDelay(10);
         }
       approximateDelay(100);
