@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2009 by The BRLTTY Developers.
+ * Copyright (C) 1995-2010 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -21,9 +21,33 @@
 #include <string.h>
 #include <errno.h>
 
-#include "misc.h"
+#include "log.h"
 
 #include "brl_driver.h"
+#include "brldefs-sk.h"
+
+BEGIN_KEY_NAME_TABLE(all)
+  KEY_NAME_ENTRY(SK_KEY_K1, "K1"),
+  KEY_NAME_ENTRY(SK_KEY_K2, "K2"),
+  KEY_NAME_ENTRY(SK_KEY_K3, "K3"),
+  KEY_NAME_ENTRY(SK_KEY_K4, "K4"),
+  KEY_NAME_ENTRY(SK_KEY_K5, "K5"),
+  KEY_NAME_ENTRY(SK_KEY_K6, "K6"),
+  KEY_NAME_ENTRY(SK_KEY_K7, "K7"),
+  KEY_NAME_ENTRY(SK_KEY_K8, "K8"),
+
+  KEY_SET_ENTRY(SK_SET_RoutingKeys, "RoutingKey"),
+END_KEY_NAME_TABLE
+
+BEGIN_KEY_NAME_TABLES(all)
+  KEY_NAME_TABLE(all),
+END_KEY_NAME_TABLES
+
+DEFINE_KEY_TABLE(all)
+
+BEGIN_KEY_TABLE_LIST
+  &KEY_TABLE_DEFINITION(all),
+END_KEY_TABLE_LIST
 
 typedef enum {
   IPT_identity,
@@ -31,23 +55,12 @@ typedef enum {
   IPT_routing
 } InputPacketType;
 
-typedef enum {
-  KEY_K1 = 0X0001,
-  KEY_K7 = 0X0002,
-  KEY_K8 = 0X0004,
-  KEY_K6 = 0X0008,
-  KEY_K5 = 0X0010,
-  KEY_K2 = 0X0200,
-  KEY_K3 = 0X0800,
-  KEY_K4 = 0X1000,
-} BrailleKeys;
-
 typedef struct {
   unsigned char bytes[24];
   unsigned char type;
 
   union {
-    BrailleKeys keys;
+    uint16_t keys;
     const unsigned char *routing;
 
     struct {
@@ -76,13 +89,9 @@ typedef struct {
 } InputOutputOperations;
 static const InputOutputOperations *io;
 
-static const int logInputPackets = 0;
-static const int logOutputPackets = 0;
-
 #define SERIAL_BAUD 9600
 static const int charactersPerSecond = SERIAL_BAUD / 10;
 
-static int routingCommand;
 static TranslationTable outputTable;
 static unsigned char textCells[40];
 
@@ -97,7 +106,7 @@ readByte (unsigned char *byte, int wait) {
 
 static int
 writeBytes (BrailleDisplay *brl, const unsigned char *buffer, size_t length) {
-  if (logOutputPackets) LogBytes(LOG_DEBUG, "Output Packet", buffer, length);
+  logOutputPacket(buffer, length);
   if (io->writeBytes(buffer, length) == -1) return 0;
   brl->writeDelay += (length * 1000 / charactersPerSecond) + 1;
   return 1;
@@ -176,7 +185,7 @@ readPacket_old (
     {
       int started = offset > 0;
       if (!readByte(&byte, started)) {
-        if (started) LogBytes(LOG_WARNING, "Partial Packet", packet->bytes, offset);
+        if (started) logPartialPacket(packet->bytes, offset);
         return 0;
       }
     }
@@ -193,7 +202,7 @@ readPacket_old (
         if ((byte & 0XE0) == 0X60) {
           template = &templateEntry_keys;
         } else {
-          LogBytes(LOG_WARNING, "Ignored Byte", &byte, 1);
+          logIgnoredByte(byte);
           template = NULL;
           continue;
         }
@@ -222,7 +231,7 @@ readPacket_old (
         if ((offset == 1) && (template->type == IPT_identity)) {
           template = alternateTemplate;
         } else {
-          LogBytes(LOG_WARNING, "Short Packet", packet->bytes, offset);
+          logShortPacket(packet->bytes, offset);
           offset = 0;
           template = NULL;
         }
@@ -233,7 +242,7 @@ readPacket_old (
 
     packet->bytes[offset++] = byte;
     if (template && (offset == template->length)) {
-      if (logInputPackets) LogBytes(LOG_DEBUG, "Input Packet", packet->bytes, offset);
+      logInputPacket(packet->bytes, offset);
 
       switch ((packet->type = template->type)) {
         case IPT_identity:
@@ -396,7 +405,7 @@ readPacket_332 (BrailleDisplay *brl, InputPacket *packet) {
     {
       int started = offset > 0;
       if (!readByte(&byte, started)) {
-        if (started) LogBytes(LOG_WARNING, "Partial Packet", packet->bytes, offset);
+        if (started) logPartialPacket(packet->bytes, offset);
         return 0;
       }
     }
@@ -409,7 +418,7 @@ readPacket_332 (BrailleDisplay *brl, InputPacket *packet) {
           break;
 
         default:
-          LogBytes(LOG_WARNING, "Ignored Byte", &byte, 1);
+          logIgnoredByte(byte);
           continue;
       }
     } else {
@@ -440,7 +449,7 @@ readPacket_332 (BrailleDisplay *brl, InputPacket *packet) {
       }
 
       if (unexpected) {
-        LogBytes(LOG_WARNING, "Short Packet", packet->bytes, offset);
+        logShortPacket(packet->bytes, offset);
         offset = 0;
         length = 0;
         goto gotByte;
@@ -449,7 +458,7 @@ readPacket_332 (BrailleDisplay *brl, InputPacket *packet) {
 
     packet->bytes[offset++] = byte;
     if (offset == length) {
-      if (logInputPackets) LogBytes(LOG_DEBUG, "Input Packet", packet->bytes, offset);
+      logInputPacket(packet->bytes, offset);
 
       switch (packet->type) {
         case IPT_identity:
@@ -573,7 +582,6 @@ static const InputOutputOperations serialOperations = {
   awaitSerialInput, readSerialBytes, writeSerialBytes
 };
 
-#ifdef ENABLE_USB_SUPPORT
 /* USB IO */
 #include "io_usb.h"
 
@@ -645,46 +653,41 @@ static const InputOutputOperations usbOperations = {
   openUsbPort, closeUsbPort,
   awaitUsbInput, readUsbBytes, writeUsbBytes
 };
-#endif /* ENABLE_USB_SUPPORT */
 
-#ifdef ENABLE_BLUETOOTH_SUPPORT
 /* Bluetooth IO */
 #include "io_bluetooth.h"
-#include "io_misc.h"
 
-static int bluetoothConnection = -1;
+static BluetoothConnection *bluetoothConnection = NULL;
 
 static int
 openBluetoothPort (const char *device) {
-  if ((bluetoothConnection = btOpenConnection(device, 1, 0)) == -1) return 0;
+  if (!(bluetoothConnection = bthOpenConnection(device, 1, 0))) return 0;
   return 1;
 }
 
 static void
 closeBluetoothPort (void) {
-  if (bluetoothConnection != -1) {
-    close(bluetoothConnection);
-    bluetoothConnection = -1;
+  if (bluetoothConnection) {
+    bthCloseConnection(bluetoothConnection);
+    bluetoothConnection = NULL;
   }
 }
 
 static int
 awaitBluetoothInput (int milliseconds) {
-  return awaitInput(bluetoothConnection, milliseconds);
+  return bthAwaitInput(bluetoothConnection, milliseconds);
 }
 
 static int
 readBluetoothBytes (unsigned char *buffer, int length, int wait) {
   const int timeout = 100;
-  size_t offset = 0;
-  return readChunk(bluetoothConnection,
-                   buffer, &offset, length,
-                   (wait? timeout: 0), timeout);
+  return bthReadData(bluetoothConnection, buffer, length,
+                     (wait? timeout: 0), timeout);
 }
 
 static int
 writeBluetoothBytes (const unsigned char *buffer, int length) {
-  int count = writeData(bluetoothConnection, buffer, length);
+  int count = bthWriteData(bluetoothConnection, buffer, length);
   if (count != length) {
     if (count == -1) {
       LogError("Bluetooth write");
@@ -700,7 +703,6 @@ static const InputOutputOperations bluetoothOperations = {
   openBluetoothPort, closeBluetoothPort,
   awaitBluetoothInput, readBluetoothBytes, writeBluetoothBytes
 };
-#endif /* ENABLE_BLUETOOTH_SUPPORT */
 
 static int
 brl_construct (BrailleDisplay *brl, char **parameters, const char *device) {
@@ -711,21 +713,11 @@ brl_construct (BrailleDisplay *brl, char **parameters, const char *device) {
   
   if (isSerialDevice(&device)) {
     io = &serialOperations;
-  } else
-
-#ifdef ENABLE_USB_SUPPORT
-  if (isUsbDevice(&device)) {
+  } else if (isUsbDevice(&device)) {
     io = &usbOperations;
-  } else
-#endif /* ENABLE_USB_SUPPORT */
-
-#ifdef ENABLE_BLUETOOTH_SUPPORT
-  if (isBluetoothDevice(&device)) {
+  } else if (isBluetoothDevice(&device)) {
     io = &bluetoothOperations;
-  } else
-#endif /* ENABLE_BLUETOOTH_SUPPORT */
-
-  {
+  } else {
     unsupportedDevice(device);
     return 0;
   }
@@ -744,11 +736,15 @@ brl_construct (BrailleDisplay *brl, char **parameters, const char *device) {
 
         brl->textColumns = response.fields.identity.size;
         brl->textRows = 1;
-        brl->helpPage = 0;
 
-        routingCommand = BRL_BLK_ROUTE;
+        {
+          const KeyTableDefinition *ktd = &KEY_TABLE_DEFINITION(all);
+
+          brl->keyBindings = ktd->bindings;
+          brl->keyNameTables = ktd->names;
+        }
+
         memset(textCells, 0XFF, brl->textColumns);
-
         return 1;
       }
     }
@@ -779,94 +775,70 @@ brl_readCommand (BrailleDisplay *brl, BRL_DriverCommandContext context) {
   InputPacket packet;
   size_t length;
 
+nextPacket:
   while ((length = protocol->readPacket(brl, &packet))) {
-    int routing = routingCommand;
-    routingCommand = BRL_BLK_ROUTE;
-
     switch (packet.type) {
-      case IPT_keys:
-#define CMD(keys,cmd) case (keys): return (cmd)
-#define BLK(keys,blk) case (keys): routingCommand = (blk); return BRL_CMD_NOOP
-        switch (packet.fields.keys) {
-          CMD(KEY_K1, BRL_CMD_FWINLT);
-          CMD(KEY_K8, BRL_CMD_FWINRT);
+      case IPT_keys: {
+        uint16_t bit = 0X1;
+        unsigned char key = 0;
 
-          CMD(KEY_K2, BRL_CMD_LNUP);
-          CMD(KEY_K3, BRL_CMD_LNDN);
-          CMD(KEY_K2 | KEY_K3, BRL_CMD_LNBEG);
+        unsigned char keys[0X10];
+        unsigned int keyCount = 0;
 
-          CMD(KEY_K6, BRL_CMD_FWINLTSKIP);
-          CMD(KEY_K7, BRL_CMD_FWINRTSKIP);
-          CMD(KEY_K6 | KEY_K7, BRL_CMD_PASTE);
-
-          CMD(KEY_K4, BRL_CMD_CSRTRK);
-          CMD(KEY_K5, BRL_CMD_RETURN);
-          CMD(KEY_K4 | KEY_K5, BRL_CMD_CSRJMP_VERT);
-
-          CMD(KEY_K6 | KEY_K2, BRL_CMD_TOP_LEFT);
-          CMD(KEY_K6 | KEY_K3, BRL_CMD_BOT_LEFT);
-          CMD(KEY_K7 | KEY_K2, BRL_CMD_TOP);
-          CMD(KEY_K7 | KEY_K3, BRL_CMD_BOT);
-
-          CMD(KEY_K4 | KEY_K2, BRL_CMD_ATTRUP);
-          CMD(KEY_K4 | KEY_K3, BRL_CMD_ATTRDN);
-          CMD(KEY_K5 | KEY_K2, BRL_CMD_PRDIFLN);
-          CMD(KEY_K5 | KEY_K3, BRL_CMD_NXDIFLN);
-          CMD(KEY_K4 | KEY_K6, BRL_CMD_PRPROMPT);
-          CMD(KEY_K4 | KEY_K7, BRL_CMD_NXPROMPT);
-          CMD(KEY_K5 | KEY_K6, BRL_CMD_PRPGRPH);
-          CMD(KEY_K5 | KEY_K7, BRL_CMD_NXPGRPH);
-
-          BLK(KEY_K1 | KEY_K8 | KEY_K2, BRL_BLK_PRINDENT);
-          BLK(KEY_K1 | KEY_K8 | KEY_K3, BRL_BLK_NXINDENT);
-          BLK(KEY_K1 | KEY_K8 | KEY_K4, BRL_BLK_SETLEFT);
-          BLK(KEY_K1 | KEY_K8 | KEY_K5, BRL_BLK_DESCCHAR);
-          BLK(KEY_K1 | KEY_K8 | KEY_K6, BRL_BLK_PRDIFCHAR);
-          BLK(KEY_K1 | KEY_K8 | KEY_K7, BRL_BLK_NXDIFCHAR);
-
-          BLK(KEY_K1 | KEY_K8 | KEY_K2 | KEY_K6, BRL_BLK_CUTBEGIN);
-          BLK(KEY_K1 | KEY_K8 | KEY_K2 | KEY_K7, BRL_BLK_CUTAPPEND);
-          BLK(KEY_K1 | KEY_K8 | KEY_K3 | KEY_K6, BRL_BLK_CUTLINE);
-          BLK(KEY_K1 | KEY_K8 | KEY_K3 | KEY_K7, BRL_BLK_CUTRECT);
-
-          CMD(KEY_K1 | KEY_K2, BRL_CMD_HELP);
-          CMD(KEY_K1 | KEY_K3, BRL_CMD_LEARN);
-          CMD(KEY_K1 | KEY_K4, BRL_CMD_PREFLOAD);
-          CMD(KEY_K1 | KEY_K5, BRL_CMD_PREFSAVE);
-          CMD(KEY_K1 | KEY_K6, BRL_CMD_PREFMENU);
-          CMD(KEY_K1 | KEY_K7, BRL_CMD_INFO);
-
-          CMD(KEY_K8 | KEY_K2, BRL_CMD_DISPMD);
-          CMD(KEY_K8 | KEY_K3, BRL_CMD_FREEZE);
-          CMD(KEY_K8 | KEY_K6, BRL_CMD_SIXDOTS);
-          CMD(KEY_K8 | KEY_K7, BRL_CMD_SKPIDLNS);
-
-          default:
-            break;
-        }
-#undef BLK
-#undef CMD
-        break;
-
-      case IPT_routing: {
-        const unsigned char *byte = packet.fields.routing;
-        unsigned char bit = 0X1;
-        unsigned char key;
-
-        for (key=0; key<brl->textColumns; key+=1) {
-          if (*byte & bit) return routing + key;
-
-          if (!(bit <<= 1)) {
-            bit = 0X1;
-            byte += 1;
+        while (++key <= 0X10) {
+          if (packet.fields.keys & bit) {
+            enqueueKeyEvent(SK_SET_NavigationKeys, key, 1);
+            keys[keyCount++] = key;
           }
+
+          bit <<= 1;
+        }
+
+        if (keyCount) {
+          do {
+            enqueueKeyEvent(SK_SET_NavigationKeys, keys[--keyCount], 0);
+          } while (keyCount);
+
+          continue;
         }
 
         break;
       }
+
+      case IPT_routing: {
+        const unsigned char *byte = packet.fields.routing;
+        unsigned char key = 0;
+
+        while (key < brl->textColumns) {
+          if (*byte) {
+            unsigned char bit = 0X1;
+
+            do {
+              if (*byte & bit) {
+                enqueueKeyEvent(SK_SET_RoutingKeys, key, 1);
+                enqueueKeyEvent(SK_SET_RoutingKeys, key, 0);
+                goto nextPacket;
+              }
+
+              key += 1;
+            } while ((bit <<= 1));
+
+            break;
+          } else {
+            key += 8;
+          }
+
+          byte += 1;
+        }
+
+        break;
+      }
+
+      default:
+        break;
     }
 
-    LogBytes(LOG_WARNING, "Unexpected Packet", packet.bytes, length);
+    logUnexpectedPacket(packet.bytes, length);
   }
   if (errno != EAGAIN) return BRL_CMD_RESTARTBRL;
 
