@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2009 by The BRLTTY Developers.
+ * Copyright (C) 1995-2010 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -34,6 +34,7 @@
 #include <langinfo.h>
 #endif /* platform-specific includes */
 
+#include "log.h"
 #include "misc.h"
 #include "lock.h"
 #include "charset.h"
@@ -404,6 +405,20 @@ truncated:
   return WEOF;
 }
 
+void
+convertStringToWchars (const char **utf8, wchar_t **characters, size_t count) {
+  while (**utf8 && (count > 1)) {
+    size_t utfs = UTF8_LEN_MAX;
+    wint_t character = convertUtf8ToWchar(utf8, &utfs);
+
+    if (character == WEOF) break;
+    *(*characters)++ = character;
+    count -= 1;
+  }
+
+  if (count) **characters = 0;
+}
+
 size_t
 convertCharToUtf8 (char c, Utf8Buffer utf8) {
   wint_t wc = convertCharToWchar(c);
