@@ -2,13 +2,14 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2008 by The BRLTTY Developers.
+ * Copyright (C) 1995-2010 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
  * This is free software, placed under the terms of the
  * GNU General Public License, as published by the Free Software
- * Foundation.  Please see the file COPYING for details.
+ * Foundation; either version 2 of the License, or (at your option) any
+ * later version. Please see the file LICENSE-GPL for details.
  *
  * Web Page: http://mielke.cc/brltty/
  *
@@ -19,14 +20,14 @@
 
 #include <string.h>
 
-#ifdef HAVE_LIBICUUC
+#ifdef HAVE_ICU
 #include <unicode/uchar.h>
-#endif /* HAVE_LIBICUUC */
+#endif /* HAVE_ICU */
 
-#include "tbl.h"
 #include "ctb.h"
 #include "ctb_internal.h"
-#include "brl.h"
+#include "unicode.h"
+#include "brldots.h"
 
 static ContractionTable *table;
 static const wchar_t *src, *srcmin, *srcmax, *cursor;
@@ -42,11 +43,16 @@ static const ContractionTableRule *currentRule;	/*pointer to current rule in tab
 #define setOffset() assignOffset(dest-destmin)
 #define clearOffset() assignOffset(CTB_NO_OFFSET)
 
+static inline const void *
+getContractionTableItem (ContractionTableOffset offset) {
+  return &table->header.bytes[offset];
+}
+
 static const ContractionTableCharacter *
 getContractionTableCharacter (wchar_t character) {
-  const ContractionTableCharacter *characters = getContractionTableItem(table->header, table->header->characters);
+  const ContractionTableCharacter *characters = getContractionTableItem(table->header.fields->characters);
   int first = 0;
-  int last = table->header->characterCount - 1;
+  int last = table->header.fields->characterCount - 1;
 
   while (first <= last) {
     int current = (first + last) / 2;
@@ -213,12 +219,12 @@ selectRule (int length) {
     wchar_t characters[2];
     characters[0] = toLowerCase(src[0]);
     characters[1] = toLowerCase(src[1]);
-    ruleOffset = table->header->rules[CTH(characters)];
+    ruleOffset = table->header.fields->rules[CTH(characters)];
     maximumLength = 0;
   }
 
   while (ruleOffset) {
-    currentRule = getContractionTableItem(table->header, ruleOffset);
+    currentRule = getContractionTableItem(ruleOffset);
     currentOpcode = currentRule->opcode;
     currentFindLength = currentRule->findlen;
 
@@ -374,7 +380,7 @@ selectRule (int length) {
             break;
         }
       }
-    }				/*Done with checking this rule */
+    }
 
     ruleOffset = currentRule->next;
   }
@@ -383,46 +389,68 @@ selectRule (int length) {
 }
 
 static int
-putBytes (const BYTE *bytes, int count) {
+putCells (const BYTE *cells, int count) {
   if (dest + count > destmax) return 0;
-  memcpy(dest, bytes, count);
+  memcpy(dest, cells, count);
   dest += count;
   return 1;
 }
 
+static int
+putCell (BYTE byte) {
+  return putCells(&byte, 1);
+}
+
 static int
 putReplace (const ContractionTableRule *rule) {
-  return putBytes((BYTE *)&rule->findrep[rule->findlen], rule->replen);
+  return putCells((BYTE *)&rule->findrep[rule->findlen], rule->replen);
 }
 
-static int
-putComputerBraille (wchar_t character) {
-  BYTE cell = convertWcharToDots(textTable, character);
-  return putBytes(&cell, 1);
-}
-
-static int
-putCharacter (wchar_t character) {
+static const ContractionTableRule *
+getAlwaysRule (wchar_t character) {
   const ContractionTableCharacter *ctc = getContractionTableCharacter(character);
   if (ctc) {
     ContractionTableOffset offset = ctc->always;
     if (offset) {
-      const ContractionTableRule *rule = getContractionTableItem(table->header, offset);
-      if (rule->replen) return putReplace(rule);
+      const ContractionTableRule *rule = getContractionTableItem(offset);
+      if (rule->replen) return rule;
     }
   }
-  return putComputerBraille(character);
+  return NULL;
+}
+
+static int
+putCharacter (wchar_t character) {
+  {
+    const ContractionTableRule *rule = getAlwaysRule(character);
+    if (rule) return putReplace(rule);
+  }
+
+  {
+#ifdef HAVE_WCHAR_H 
+    const wchar_t substitute = UNICODE_REPLACEMENT_CHARACTER;
+#else /* HAVE_WCHAR_H */
+    const wchar_t substitute = 0X1A; /* ASCII SUB (CTRL-Z) */
+#endif /* HAVE_WCHAR_H */
+
+    if (character != substitute) {
+      const ContractionTableRule *rule = getAlwaysRule(substitute);
+      if (rule) return putReplace(rule);
+    }
+  }
+
+  return putCell(BRL_DOT1 | BRL_DOT2 | BRL_DOT3 | BRL_DOT4 | BRL_DOT5 | BRL_DOT6 | BRL_DOT7 | BRL_DOT8);
 }
 
 static int
 putSequence (ContractionTableOffset offset) {
-  const BYTE *sequence = getContractionTableItem(table->header, offset);
-  return putBytes(sequence+1, *sequence);
+  const BYTE *sequence = getContractionTableItem(offset);
+  return putCells(sequence+1, *sequence);
 }
 
 static void
 findLineBreakOpportunities (unsigned char *opportunities, const wchar_t *characters, size_t length) {
-#ifdef HAVE_LIBICUUC
+#ifdef HAVE_ICU
   /* UAX #14: Line Breaking Properties
    * http://unicode.org/reports/tr14/
    * Section 6: Line Breaking Algorithm
@@ -819,7 +847,7 @@ findLineBreakOpportunities (unsigned char *opportunities, const wchar_t *charact
       }
     }
   }
-#else /* HAVE_LIBICUUC */
+#else /* HAVE_ICU */
   int wasSpace = 0;
   int index;
 
@@ -828,7 +856,7 @@ findLineBreakOpportunities (unsigned char *opportunities, const wchar_t *charact
     opportunities[index] = wasSpace && !isSpace;
     wasSpace = isSpace;
   }
-#endif /* HAVE_LIBICUUC */
+#endif /* HAVE_ICU */
 }
 
 int
@@ -888,10 +916,10 @@ contractText (
         continue;
       }
 
-      if (table->header->numberSign && (previousOpcode != CTO_MidNum) &&
+      if (table->header.fields->numberSign && (previousOpcode != CTO_MidNum) &&
           !testCharacter(before, CTC_Digit) && testCharacter(*src, CTC_Digit)) {
-        if (!putSequence(table->header->numberSign)) break;
-      } else if (table->header->englishLetterSign && testCharacter(*src, CTC_Letter)) {
+        if (!putSequence(table->header.fields->numberSign)) break;
+      } else if (table->header.fields->englishLetterSign && testCharacter(*src, CTC_Letter)) {
         if ((currentOpcode == CTO_Contraction) ||
             ((currentOpcode != CTO_EndNum) && testCharacter(before, CTC_Digit)) ||
             (testCharacter(*src, CTC_Letter) &&
@@ -901,23 +929,23 @@ contractText (
              (((src + 1) == srcmax) ||
               testCharacter(src[1], CTC_Space) ||
               (testCharacter(src[1], CTC_Punctuation) && (src[1] != '.') && (src[1] != '\''))))) {
-          if (!putSequence(table->header->englishLetterSign)) break;
+          if (!putSequence(table->header.fields->englishLetterSign)) break;
         }
       }
 
       if (testCharacter(*src, CTC_UpperCase)) {
         if (!testCharacter(before, CTC_UpperCase)) {
-          if (table->header->beginCapitalSign &&
+          if (table->header.fields->beginCapitalSign &&
               (src + 1 < srcmax) && testCharacter(src[1], CTC_UpperCase)) {
-            if (!putSequence(table->header->beginCapitalSign)) break;
-          } else if (table->header->capitalSign) {
-            if (!putSequence(table->header->capitalSign)) break;
+            if (!putSequence(table->header.fields->beginCapitalSign)) break;
+          } else if (table->header.fields->capitalSign) {
+            if (!putSequence(table->header.fields->capitalSign)) break;
           }
         }
       } else if (testCharacter(*src, CTC_LowerCase)) {
-        if (table->header->endCapitalSign && (src - 2 >= srcmin) &&
+        if (table->header.fields->endCapitalSign && (src - 2 >= srcmin) &&
             testCharacter(src[-1], CTC_UpperCase) && testCharacter(src[-2], CTC_UpperCase)) {
-          if (!putSequence(table->header->endCapitalSign)) break;
+          if (!putSequence(table->header.fields->endCapitalSign)) break;
         }
       }
 
@@ -1009,7 +1037,8 @@ contractText (
         }
       }
     } else {
-      if (!putComputerBraille(*src)) break;
+      currentOpcode = CTO_Always;
+      if (!putCharacter(*src)) break;
       src += 1;
     }
 
@@ -1026,7 +1055,7 @@ contractText (
     if ((dest == destmin) || dest[-1]) {
       previousOpcode = currentOpcode;
     }
-  }				/*end of translation loop */
+  }
 
 done:
   if (src < srcmax) {
@@ -1065,4 +1094,4 @@ done:
   *inputLength = src - srcmin;
   *outputLength = dest - destmin;
   return 1;
-}				/*translation completed */
+}
