@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2009 by The BRLTTY Developers.
+ * Copyright (C) 1995-2010 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -28,7 +28,9 @@
 
 #include "program.h"
 #include "options.h"
-#include "misc.h"
+#include "log.h"
+#include "file.h"
+#include "parse.h"
 #include "charset.h"
 #include "unicode.h"
 //#include "brl.h"
