@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2009 by The BRLTTY Developers.
+ * Copyright (C) 1995-2010 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -23,12 +23,21 @@
 extern "C" {
 #endif /* __cplusplus */
 
+typedef struct BluetoothConnectionStruct BluetoothConnection;
+
+extern void bthForgetConnectErrors (void);
+
+extern BluetoothConnection *bthOpenConnection (const char *address, uint8_t channel, int force);
+extern void bthCloseConnection (BluetoothConnection *connection);
+extern int bthAwaitInput (BluetoothConnection *connection, int milliseconds);
+extern ssize_t bthReadData (
+  BluetoothConnection *connection, void *buffer, size_t size,
+  int initialTimeout, int subsequentTimeout
+);
+extern ssize_t bthWriteData (BluetoothConnection *conection, const void *buffer, size_t size);
+
 extern int isBluetoothDevice (const char **path);
 
-extern void btForgetConnectErrors (void);
-
-extern int btOpenConnection (const char *address, unsigned char channel, int force);
-
 #ifdef __cplusplus
 }
 #endif /* __cplusplus */
