@@ -2,13 +2,14 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2007 by The BRLTTY Developers.
+ * Copyright (C) 1995-2010 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
  * This is free software, placed under the terms of the
  * GNU General Public License, as published by the Free Software
- * Foundation.  Please see the file COPYING for details.
+ * Foundation; either version 2 of the License, or (at your option) any
+ * later version. Please see the file LICENSE-GPL for details.
  *
  * Web Page: http://mielke.cc/brltty/
  *
@@ -22,6 +23,8 @@
 extern "C" {
 #endif /* __cplusplus */
 
+#include "pid.h"
+
 extern const char *programPath;
 extern const char *programName;
 
@@ -31,6 +34,8 @@ extern void makeProgramBanner (char *buffer, size_t size);
 extern void fixInstallPaths (char **const *paths);
 extern void fixInstallPath (char **path);
 
+extern int createPidFile (const char *path, ProcessIdentifier pid);
+
 #ifdef __cplusplus
 }
 #endif /* __cplusplus */
