@@ -2,13 +2,14 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2005 by The BRLTTY Team. All rights reserved.
+ * Copyright (C) 1995-2010 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
  * This is free software, placed under the terms of the
  * GNU General Public License, as published by the Free Software
- * Foundation.  Please see the file COPYING for details.
+ * Foundation; either version 2 of the License, or (at your option) any
+ * later version. Please see the file LICENSE-GPL for details.
  *
  * Web Page: http://mielke.cc/brltty/
  *
@@ -23,15 +24,12 @@ extern "C" {
 #endif /* __cplusplus */
 
 #include "scr_base.h"
-#include "help.h"
 
 typedef struct {
   BaseScreen base;
-  int (*open) (const char *);
-  void (*close) (void);		   /* called once to close the help screen */
-  void (*setPageNumber) (short);
-  short (*getPageNumber) (void);
-  short (*getPageCount) (void);
+  int (*construct) (void);
+  void (*destruct) (void);
+  int (*addLine) (const wchar_t *line);
 } HelpScreen;
 
 extern void initializeHelpScreen (HelpScreen *);
