@@ -2,13 +2,14 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2005 by The BRLTTY Team. All rights reserved.
+ * Copyright (C) 1995-2010 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
  * This is free software, placed under the terms of the
  * GNU General Public License, as published by the Free Software
- * Foundation.  Please see the file COPYING for details.
+ * Foundation; either version 2 of the License, or (at your option) any
+ * later version. Please see the file LICENSE-GPL for details.
  *
  * Web Page: http://mielke.cc/brltty/
  *
@@ -23,18 +24,25 @@ extern "C" {
 #endif /* __cplusplus */
 
 #include "scr_base.h"
+#include "driver.h"
 
 typedef struct {
   BaseScreen base;
-  int (*prepare) (char **parameters);
-  int (*open) (void);
-  int (*setup) (void);
-  void (*close) (void);
-  int (*uservt) (int number);
+  int (*processParameters) (char **parameters);
+  int (*construct) (void);
+  void (*destruct) (void);
+  int (*userVirtualTerminal) (int number);
 } MainScreen;
 
 extern void initializeMainScreen (MainScreen *);
 
+struct ScreenDriverStruct {
+  DRIVER_DEFINITION_DECLARATION;
+  const char *const *parameters;
+
+  void (*initialize) (MainScreen *main);		/* initialize speech device */
+};
+
 #ifdef __cplusplus
 }
 #endif /* __cplusplus */
