@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2009 by The BRLTTY Developers.
+ * Copyright (C) 1995-2010 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -140,7 +140,10 @@ typedef int SerialLines;
 
 #include "io_serial.h"
 #include "io_misc.h"
-#include "misc.h"
+#include "log.h"
+#include "device.h"
+#include "parse.h"
+#include "timing.h"
 
 typedef void * (*FlowControlProc) (void *arg);
 
