@@ -2,13 +2,14 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2007 by The BRLTTY Developers.
+ * Copyright (C) 1995-2010 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
  * This is free software, placed under the terms of the
  * GNU General Public License, as published by the Free Software
- * Foundation.  Please see the file COPYING for details.
+ * Foundation; either version 2 of the License, or (at your option) any
+ * later version. Please see the file LICENSE-GPL for details.
  *
  * Web Page: http://mielke.cc/brltty/
  *
@@ -23,7 +24,10 @@
 #include <fcntl.h>
 #include <sys/stat.h>
 
-#include "misc.h"
+#include "log.h"
+#include "file.h"
+#include "parse.h"
+#include "charset.h"
 #include "drivers.h"
 #include "spk.h"
 #include "spk.auto.h"
@@ -37,48 +41,24 @@
 #include "spk_driver.h"
 
 static int
-spk_construct (char **parameters) {
+spk_construct (SpeechSynthesizer *spk, char **parameters) {
   return 1;
 }
 
 static void
-spk_destruct (void) {
+spk_destruct (SpeechSynthesizer *spk) {
 }
 
 static void
-spk_say (const unsigned char *buffer, int length) {
+spk_say (SpeechSynthesizer *spk, const unsigned char *text, size_t length, size_t count, const unsigned char *attributes) {
 }
 
 static void
-spk_mute (void) {
+spk_mute (SpeechSynthesizer *spk) {
 }
 
 const SpeechDriver *speech = &noSpeech;
 
-static const float spkRateTable[] = {
-  0.3333,
-  0.3720,
-  0.4152,
-  0.4635,
-  0.5173,
-  0.5774,
-  0.6444,
-  0.7192,
-  0.8027,
-  0.8960,
-  1.0000,
-  1.1161,
-  1.2457,
-  1.3904,
-  1.5518,
-  1.7320,
-  1.9332,
-  2.1577,
-  2.4082,
-  2.6879,
-  3.0000
-};
-
 int
 haveSpeechDriver (const char *code) {
   return haveDriver(code, SPEECH_DRIVER_CODES, driverTable);
@@ -112,30 +92,134 @@ identifySpeechDrivers (int full) {
 }
 
 void
-sayString (const char *string, int mute) {
-  if (mute) speech->mute();
-  speech->say((unsigned char *)string, strlen(string));
+initializeSpeechSynthesizer (SpeechSynthesizer *spk) {
+  spk->data = NULL;
+}
+
+void
+sayCharacters (SpeechSynthesizer *spk, const char *characters, size_t count, int mute) {
+  if (count) {
+    unsigned char bytes[(count * UTF8_LEN_MAX) + 1];
+    unsigned char *b = bytes;
+
+    {
+      int i;
+      for (i=0; i<count; i+=1) {
+        Utf8Buffer utf8;
+        size_t utfs = convertCharToUtf8(characters[i], utf8);
+
+        if (utfs) {
+          memcpy(b, utf8, utfs);
+          b += utfs;
+        } else {
+          *b++ = ' ';
+        }
+      }
+
+      *b = 0;
+    }
+
+    if (mute) speech->mute(spk);
+    speech->say(spk, bytes, b-bytes, count, NULL);
+  }
+}
+
+void
+sayString (SpeechSynthesizer *spk, const char *string, int mute) {
+  sayCharacters(spk, string, strlen(string), mute);
 }
 
 static void
-saySpeechSetting (int setting, const char *name) {
+saySpeechSetting (SpeechSynthesizer *spk, int setting, const char *name) {
   char phrase[0X40];
   snprintf(phrase, sizeof(phrase), "%s %d", name, setting);
-  sayString(phrase, 1);
+  sayString(spk, phrase, 1);
+}
+
+static unsigned int
+getIntegerSetting (unsigned char setting, unsigned char internal, unsigned int external) {
+  return rescaleInteger(setting, internal, external);
 }
 
 void
-setSpeechRate (int setting, int say) {
+setSpeechRate (SpeechSynthesizer *spk, int setting, int say) {
   LogPrint(LOG_DEBUG, "setting speech rate: %d", setting);
-  speech->rate(spkRateTable[setting]);
-  if (say) saySpeechSetting(setting, "rate");
+  speech->rate(spk, setting);
+  if (say) saySpeechSetting(spk, setting, "rate");
+}
+
+unsigned int
+getIntegerSpeechRate (unsigned char setting, unsigned int normal) {
+  return getIntegerSetting(setting, SPK_RATE_DEFAULT, normal);
+}
+
+float
+getFloatSpeechRate (unsigned char setting) {
+  static const float spkRateTable[] = {
+    0.3333,
+    0.3720,
+    0.4152,
+    0.4635,
+    0.5173,
+    0.5774,
+    0.6444,
+    0.7192,
+    0.8027,
+    0.8960,
+    1.0000,
+    1.1161,
+    1.2457,
+    1.3904,
+    1.5518,
+    1.7320,
+    1.9332,
+    2.1577,
+    2.4082,
+    2.6879,
+    3.0000
+  };
+
+  return spkRateTable[setting];
 }
 
 void
-setSpeechVolume (int setting, int say) {
+setSpeechVolume (SpeechSynthesizer *spk, int setting, int say) {
   LogPrint(LOG_DEBUG, "setting speech volume: %d", setting);
-  speech->volume((float)setting / (float)SPK_DEFAULT_VOLUME);
-  if (say) saySpeechSetting(setting, "volume");
+  speech->volume(spk, setting);
+  if (say) saySpeechSetting(spk, setting, "volume");
+}
+
+unsigned int
+getIntegerSpeechVolume (unsigned char setting, unsigned int normal) {
+  return getIntegerSetting(setting, SPK_VOLUME_DEFAULT, normal);
+}
+
+float
+getFloatSpeechVolume (unsigned char setting) {
+  return (float)setting / (float)SPK_VOLUME_DEFAULT;
+}
+
+void
+setSpeechPitch (SpeechSynthesizer *spk, int setting, int say) {
+  LogPrint(LOG_DEBUG, "setting speech pitch: %d", setting);
+  speech->pitch(spk, setting);
+  if (say) saySpeechSetting(spk, setting, "pitch");
+}
+
+unsigned int
+getIntegerSpeechPitch (unsigned char setting, unsigned int normal) {
+  return getIntegerSetting(setting, SPK_PITCH_DEFAULT, normal);
+}
+
+float
+getFloatSpeechPitch (unsigned char setting) {
+  return (float)setting / (float)SPK_PITCH_DEFAULT;
+}
+
+void
+setSpeechPunctuation (SpeechSynthesizer *spk, SpeechPunctuation setting, int say) {
+  LogPrint(LOG_DEBUG, "setting speech punctuation: %d", setting);
+  speech->punctuation(spk, setting);
 }
 
 static char *speechFifoPath = NULL;
@@ -226,17 +310,17 @@ openSpeechFifo (const char *directory, const char *path) {
 }
 
 void
-processSpeechFifo (void) {
+processSpeechFifo (SpeechSynthesizer *spk) {
 #ifdef __MINGW32__
   if (speechFifoHandle != INVALID_HANDLE_VALUE) {
     if (speechFifoConnected ||
         (speechFifoConnected = ConnectNamedPipe(speechFifoHandle, NULL)) ||
         (speechFifoConnected = (GetLastError() == ERROR_PIPE_CONNECTED))) {
-      unsigned char buffer[0X1000];
+      char buffer[0X1000];
       DWORD count;
 
       if (ReadFile(speechFifoHandle, buffer, sizeof(buffer), &count, NULL)) {
-        if (count) speech->say(buffer, count);
+        if (count) sayCharacters(spk, buffer, count, 0);
       } else {
         DWORD error = GetLastError();
 
@@ -252,9 +336,9 @@ processSpeechFifo (void) {
   }
 #else /* __MINGW32__ */
   if (speechFifoDescriptor != -1) {
-    unsigned char buffer[0X1000];
+    char buffer[0X1000];
     int count = read(speechFifoDescriptor, buffer, sizeof(buffer));
-    if (count > 0) speech->say(buffer, count);
+    if (count > 0) sayCharacters(spk, buffer, count, 0);
   }
 #endif /* __MINGW32__ */
 }
