@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2009 by The BRLTTY Developers.
+ * Copyright (C) 1995-2010 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -29,6 +29,8 @@
 
 #include "program.h"
 #include "options.h"
+#include "log.h"
+#include "file.h"
 #include "misc.h"
 #include "brldots.h"
 #include "charset.h"
@@ -223,11 +225,11 @@ writeCharacter_native (FILE *file, wchar_t character, unsigned char dots, const
   if (fprintf(file, "char ") == EOF) goto error;
 
   if (value < 0X100) {
-    if (fprintf(file, "\\x%02X", value) == EOF) goto error;
+    if (fprintf(file, "\\x%02" PRIX32, value) == EOF) goto error;
   } else if (value < 0X10000) {
-    if (fprintf(file, "\\u%04X", value) == EOF) goto error;
+    if (fprintf(file, "\\u%04" PRIX32, value) == EOF) goto error;
   } else {
-    if (fprintf(file, "\\U%08X", value) == EOF) goto error;
+    if (fprintf(file, "\\U%08" PRIX32, value) == EOF) goto error;
   }
 
   if (fprintf(file, "\t") == EOF) goto error;
@@ -616,10 +618,25 @@ convertTable (void) {
 
 #else /* standard input/output */
 #warning curses package either unspecified or unsupported
+
+#define refresh() fflush(stdout)
 #define printw printf
 #define erase() printf("\r\n\v")
-#define refresh() fflush(stdout)
 #define beep() printf("\a")
+
+static int inputAttributesChanged;
+
+#if defined(__MINGW32__)
+#define STDIN_HANDLE ((HANDLE)_get_osfhandle(STDIN_FILENO))
+static long inputConsoleMode;
+
+#undef beep
+#define beep() MessageBeep(MB_ICONWARNING)
+
+#else /* termios */
+#include <termios.h>
+static struct termios inputTerminalAttributes;
+#endif /* input terminal definitions */
 #endif /* curses package */
 
 typedef struct {
@@ -686,7 +703,7 @@ static int
 getCharacter (EditTableData *etd, wchar_t *character) {
   if (etd->charset) {
     wint_t wc = convertCharToWchar(etd->character.byte);
-    if (wc == WEOF) return 0;;
+    if (wc == WEOF) return 0;
     *character = wc;
   } else {
     *character = etd->character.unicode;
@@ -784,6 +801,27 @@ printCharacterString (const wchar_t *wcs) {
   }
 }
 
+static void
+printNavigationPair (
+  const char *key1, const char *preposition1,
+  const char *key2, const char *preposition2,
+  const char *adjective, const char *noun,
+  int width
+) {
+  int length = 0;
+
+  if (*key1 || *key2) {
+    const char *separator = (*key1 && *key2)? "/": "";
+
+    printw("%s%s%s: %s%s%s %s %s%n",
+           key1, separator, key2,
+           (*key1? preposition1: ""), separator, (*key2? preposition2: ""),
+           adjective, noun, &length);
+  }
+
+  if (width > length) printw("%*s", width-length, "");
+}
+
 static int
 updateCharacterDescription (EditTableData *etd) {
   int ok = 0;
@@ -801,51 +839,129 @@ updateCharacterDescription (EditTableData *etd) {
     erase();
 
 #if defined(USE_CURSES)
-    printw("Left/Right: previous/next %s character\n",
-           etd->charset? etd->charset: "unicode");
-    printw("Up/Down: previous/next defined character\n");
-    printw("Home/End: first/last defined character\n");
+#define KEY_FIRST_ACTUAL_CHARACTER ""
+#define KEY_LAST_ACTUAL_CHARACTER ""
+#define KEY_PREVIOUS_ACTUAL_CHARACTER "Left"
+#define KEY_NEXT_ACTUAL_CHARACTER "Right"
+#define KEY_FIRST_DEFINED_CHARACTER "Home"
+#define KEY_LAST_DEFINED_CHARACTER "End"
+#define KEY_PREVIOUS_DEFINED_CHARACTER "Up"
+#define KEY_NEXT_DEFINED_CHARACTER "Down"
+#define KEY_TOGGLE_DOT1 "F4"
+#define KEY_TOGGLE_DOT2 "F3"
+#define KEY_TOGGLE_DOT3 "F2"
+#define KEY_TOGGLE_DOT4 "F5"
+#define KEY_TOGGLE_DOT5 "F6"
+#define KEY_TOGGLE_DOT6 "F7"
+#define KEY_TOGGLE_DOT7 "F1"
+#define KEY_TOGGLE_DOT8 "F8"
+#define KEY_TOGGLE_CHARACTER "F9"
+#define KEY_ALTERNATE_CHARACTER "F10"
+#define KEY_SAVE_TABLE "F11"
+#define KEY_EXIT_EDITOR "F12"
+#else /* standard input/output */
+#define KEY_FIRST_ACTUAL_CHARACTER "^S"
+#define KEY_LAST_ACTUAL_CHARACTER "^G"
+#define KEY_PREVIOUS_ACTUAL_CHARACTER "^D"
+#define KEY_NEXT_ACTUAL_CHARACTER "^F"
+#define KEY_FIRST_DEFINED_CHARACTER "^H"
+#define KEY_LAST_DEFINED_CHARACTER "^L"
+#define KEY_PREVIOUS_DEFINED_CHARACTER "^J"
+#define KEY_NEXT_DEFINED_CHARACTER "^K"
+#define KEY_TOGGLE_DOT1 "^R"
+#define KEY_TOGGLE_DOT2 "^E"
+#define KEY_TOGGLE_DOT3 "^W"
+#define KEY_TOGGLE_DOT4 "^U"
+#define KEY_TOGGLE_DOT5 "^I"
+#define KEY_TOGGLE_DOT6 "^O"
+#define KEY_TOGGLE_DOT7 "^Q"
+#define KEY_TOGGLE_DOT8 "^P"
+#define KEY_TOGGLE_CHARACTER "^T"
+#define KEY_ALTERNATE_CHARACTER "^Y"
+#define KEY_SAVE_TABLE "^A"
+#define KEY_EXIT_EDITOR "^Z"
+#endif /* key definitions */
 
-#define DOT(n) printw("F%u: %s dot %u  ", n, ((dots & BRLAPI_DOT##n)? "lower": "raise"), n)
+    {
+      const char *first = "first";
+      const char *last = "last";
+      const char *previous = "prev";
+      const char *next = "next";
+
+      const char *actual = etd->charset? etd->charset: "unicode";
+      const char *defined = "defined";
+
+      const char *character = "char";
+      const int width = 38;
+
+      printNavigationPair(KEY_PREVIOUS_ACTUAL_CHARACTER, previous,
+                          KEY_NEXT_ACTUAL_CHARACTER, next,
+                          actual, character, width);
+      printNavigationPair(KEY_PREVIOUS_DEFINED_CHARACTER, previous,
+                          KEY_NEXT_DEFINED_CHARACTER, next,
+                          defined, character, 0);
+      printw("\n");
+
+      printNavigationPair(KEY_FIRST_ACTUAL_CHARACTER, first,
+                          KEY_LAST_ACTUAL_CHARACTER, last,
+                          actual, character, width);
+      printNavigationPair(KEY_FIRST_DEFINED_CHARACTER, first,
+                          KEY_LAST_DEFINED_CHARACTER, last,
+                          defined, character, 0);
+      printw("\n");
+    }
+    printw("\n");
+
+#define DOT(n) printw("%s: %s dot %u    ", KEY_TOGGLE_DOT##n, ((dots & BRLAPI_DOT##n)? "lower": "raise"), n)
     DOT(1);
-    DOT(5);
-    printw("F9: %s",
+    DOT(4);
+    printw("%s: %s", KEY_TOGGLE_CHARACTER,
            !gotCharacter? "":
-           !gotDots? "define (as empty cell)":
+           !gotDots? "define character (empty cell)":
            dots? "clear all dots":
            "undefine character");
     printw("\n");
 
     DOT(2);
-    DOT(6);
-    printw("F10:");
+    DOT(5);
+    printw("%s:", KEY_ALTERNATE_CHARACTER);
     {
-      static const char *label_SwitchCase = "switch case";
-      const char *label = NULL;
+      const char *lower = "lowercase";
+      const char *upper = "uppercase";
+      const char *alternate = NULL;
+
       if (etd->charset) {
-        if (isalpha(etd->character.byte)) label = label_SwitchCase;
+        if (isupper(etd->character.byte)) {
+          alternate = lower;
+        } else if (islower(etd->character.byte)) {
+          alternate = upper;
+        }
       } else {
-        if (iswalpha(etd->character.unicode)) label = label_SwitchCase;
+        if (iswupper(etd->character.unicode)) {
+          alternate = lower;
+        } else if (iswlower(etd->character.unicode)) {
+          alternate = upper;
+        }
       }
-      if (label) printw(" %s", label);
+
+      if (alternate) printw(" switch to %s equivalent", alternate);
     }
     printw("\n");
 
     DOT(3);
-    DOT(7);
-    printw("F11: %s", etd->updated? "save table": "");
+    DOT(6);
+    printw("%s: %s", KEY_SAVE_TABLE,
+           etd->updated? "save table": "");
     printw("\n");
 
-    DOT(4);
+    DOT(7);
     DOT(8);
-    printw("F12: exit table editor");
+    printw("%s: exit table editor", KEY_EXIT_EDITOR);
     if (etd->updated) printw(" (unsaved changes)");
     printw("\n");
 #undef DOT
 
     printw("\n");
-#else /* standard input/output */
-#endif /* write header */
 
     if (etd->charset) {
       printw("%02X: %s\n", etd->character.byte, etd->charset);
@@ -904,7 +1020,7 @@ setPreviousActualCharacter (EditTableData *etd) {
   if (etd->charset) {
     etd->character.byte = (etd->character.byte - 1) & CHARSET_BYTE_MAXIMUM;
   } else {
-    etd->character.unicode = (etd->character.unicode - 1) & UNICODE_CHARACTER_MASK;
+    etd->character.unicode = (etd->character.unicode - 1) & WCHAR_MAX;
   }
 }
 
@@ -913,7 +1029,7 @@ setNextActualCharacter (EditTableData *etd) {
   if (etd->charset) {
     etd->character.byte = (etd->character.byte + 1) & CHARSET_BYTE_MAXIMUM;
   } else {
-    etd->character.unicode = (etd->character.unicode + 1) & UNICODE_CHARACTER_MASK;
+    etd->character.unicode = (etd->character.unicode + 1) & WCHAR_MAX;
   }
 }
 
@@ -931,7 +1047,7 @@ setLastActualCharacter (EditTableData *etd) {
   if (etd->charset) {
     etd->character.byte = CHARSET_BYTE_MAXIMUM;
   } else {
-    etd->character.unicode = UNICODE_CHARACTER_MASK;
+    etd->character.unicode = WCHAR_MAX;
   }
 }
 
@@ -1196,7 +1312,7 @@ doKeyboardCommand (EditTableData *etd) {
         break;
 
       case KEY_DOWN:
-        if (!setNextDefinedCharacter(etd)) beep();;
+        if (!setNextDefinedCharacter(etd)) beep();
         break;
 
       case KEY_HOME:
@@ -1208,31 +1324,31 @@ doKeyboardCommand (EditTableData *etd) {
         break;
 
       case KEY_F(1):
-        if (!toggleDot(etd, BRLAPI_DOT1)) beep();
+        if (!toggleDot(etd, BRLAPI_DOT7)) beep();
         break;
 
       case KEY_F(2):
-        if (!toggleDot(etd, BRLAPI_DOT2)) beep();
-        break;
-
-      case KEY_F(3):
         if (!toggleDot(etd, BRLAPI_DOT3)) beep();
         break;
 
+      case KEY_F(3):
+        if (!toggleDot(etd, BRLAPI_DOT2)) beep();
+        break;
+
       case KEY_F(4):
-        if (!toggleDot(etd, BRLAPI_DOT4)) beep();
+        if (!toggleDot(etd, BRLAPI_DOT1)) beep();
         break;
 
       case KEY_F(5):
-        if (!toggleDot(etd, BRLAPI_DOT5)) beep();
+        if (!toggleDot(etd, BRLAPI_DOT4)) beep();
         break;
 
       case KEY_F(6):
-        if (!toggleDot(etd, BRLAPI_DOT6)) beep();
+        if (!toggleDot(etd, BRLAPI_DOT5)) beep();
         break;
 
       case KEY_F(7):
-        if (!toggleDot(etd, BRLAPI_DOT7)) beep();
+        if (!toggleDot(etd, BRLAPI_DOT6)) beep();
         break;
 
       case KEY_F(8):
@@ -1265,8 +1381,123 @@ doKeyboardCommand (EditTableData *etd) {
 #endif /* USE_FUNC_GET_WCH */
 #else /* standard input/output */
 #define IS_UNICODE_CHARACTER
+  int handled = 1;
   wint_t ch = fgetwc(stdin);
   if (ch == WEOF) return 0;
+
+  switch (ch) {
+    case 0X1B: /* escape */
+      return 0;
+
+    case 0X11: /* CTRL-Q */
+      if (!toggleDot(etd, BRLAPI_DOT7)) beep();
+      break;
+
+    case 0X17: /* CTRL-W */
+      if (!toggleDot(etd, BRLAPI_DOT3)) beep();
+      break;
+
+    case 0X05: /* CTRL-E */
+      if (!toggleDot(etd, BRLAPI_DOT2)) beep();
+      break;
+
+    case 0X12: /* CTRL-R */
+      if (!toggleDot(etd, BRLAPI_DOT1)) beep();
+      break;
+
+    case 0X14: /* CTRL-T */
+      if (!toggleCharacter(etd)) beep();
+      break;
+
+    case 0X19: /* CTRL-Y */
+      if (!setAlternateCharacter(etd)) beep();
+      break;
+
+    case 0X15: /* CTRL-U */
+      if (!toggleDot(etd, BRLAPI_DOT4)) beep();
+      break;
+
+    case 0X09: /* CTRL-I */
+      if (!toggleDot(etd, BRLAPI_DOT5)) beep();
+      break;
+
+    case 0X0F: /* CTRL-O */
+      if (!toggleDot(etd, BRLAPI_DOT6)) beep();
+      break;
+
+    case 0X10: /* CTRL-P */
+      if (!toggleDot(etd, BRLAPI_DOT8)) beep();
+      break;
+
+    case 0X01: /* CTRL-A */
+      if (!(etd->updated && saveTable(etd))) beep();
+      break;
+
+    case 0X13: /* CTRL-S */
+      setFirstActualCharacter(etd);
+      break;
+
+    case 0X04: /* CTRL-D */
+      setPreviousActualCharacter(etd);
+      break;
+
+    case 0X06: /* CTRL-F */
+      setNextActualCharacter(etd);
+      break;
+
+    case 0X07: /* CTRL-G */
+      setLastActualCharacter(etd);
+      break;
+
+    case 0X08: /* CTRL-H */
+      if (!setFirstDefinedCharacter(etd)) beep();
+      break;
+
+    case 0X0A: /* CTRL-J */
+      if (!setPreviousDefinedCharacter(etd)) beep();
+      break;
+
+    case 0X0B: /* CTRL-K */
+      if (!setNextDefinedCharacter(etd)) beep();
+      break;
+
+    case 0X0C: /* CTRL-L */
+      if (!setLastDefinedCharacter(etd)) beep();
+      break;
+
+    case 0X1A: /* CTRL-Z */
+      return 0;
+
+    case 0X18: /* CTRL-X */
+      beep();
+      break;
+
+    case 0X03: /* CTRL-C */
+      beep();
+      break;
+
+    case 0X16: /* CTRL-V */
+      beep();
+      break;
+
+    case 0X02: /* CTRL-B */
+      beep();
+      break;
+
+    case 0X0E: /* CTRL-N */
+      beep();
+      break;
+
+    case 0X0D: /* CTRL-M */
+      beep();
+      break;
+
+    default:
+      handled = 0;
+      break;
+  }
+
+  if (!handled)
 #endif /* read character */
 
   {
@@ -1469,6 +1700,40 @@ editTable (void) {
     nonl();
     intrflush(stdscr, FALSE);
 #else /* standard input/output */
+    setvbuf(stdin, NULL, _IONBF, 0);
+    inputAttributesChanged = 0;
+
+#if defined(__MINGW32__)
+    if (GetConsoleMode(STDIN_HANDLE, &inputConsoleMode)) {
+      long newConsoleMode = inputConsoleMode;
+      newConsoleMode &= ~(ENABLE_LINE_INPUT | ENABLE_ECHO_INPUT | ENABLE_PROCESSED_INPUT);
+
+      if (SetConsoleMode(STDIN_HANDLE, newConsoleMode)) {
+        inputAttributesChanged = 1;
+      }
+    }
+#else /* termios */
+    if (tcgetattr(STDIN_FILENO, &inputTerminalAttributes) != -1) {
+      struct termios newAttributes = inputTerminalAttributes;
+
+      newAttributes.c_iflag &= ~(IGNBRK | BRKINT | PARMRK | ISTRIP | INLCR | IGNCR | ICRNL | IXON);
+      newAttributes.c_lflag &= ~(ECHO | ECHONL | ICANON | ISIG | IEXTEN);
+      newAttributes.c_cflag &= ~(CSIZE | PARENB);
+      newAttributes.c_cflag |= CS8;
+
+      {
+        int i;
+        for (i=0; i<NCCS; i+=1) newAttributes.c_cc[i] = _POSIX_VDISABLE;
+      }
+
+      newAttributes.c_cc[VTIME] = 0;
+      newAttributes.c_cc[VMIN] = 1;
+
+      if (tcsetattr(STDIN_FILENO, TCSAFLUSH, &newAttributes) != -1) {
+        inputAttributesChanged = 1;
+      };
+    }
+#endif /* input terminal initialization */
 #endif /* initialize keyboard and screen */
 
     etd.charset = *opt_charset? opt_charset: NULL;
@@ -1478,6 +1743,7 @@ editTable (void) {
       fd_set set;
       FD_ZERO(&set);
 
+#ifndef __MINGW32__
       {
         int maximumFileDescriptor = STDIN_FILENO;
         FD_SET(STDIN_FILENO, &set);
@@ -1491,13 +1757,15 @@ editTable (void) {
         select(maximumFileDescriptor+1, &set, NULL, NULL, NULL);
       }
 
-      if (FD_ISSET(STDIN_FILENO, &set))
-        if (!doKeyboardCommand(&etd))
-          break;
+      if (haveBrailleDisplay(&etd) && FD_ISSET(etd.brlapiFileDescriptor, &set)) {
+        if (!doBrailleCommand(&etd)) break;
+      }
 
-      if (haveBrailleDisplay(&etd) && FD_ISSET(etd.brlapiFileDescriptor, &set))
-        if (!doBrailleCommand(&etd))
-          break;
+      if (FD_ISSET(STDIN_FILENO, &set))
+#endif /* __MINGW32__ */
+      {
+        if (!doKeyboardCommand(&etd)) break;
+      }
     }
 
     erase();
@@ -1506,6 +1774,13 @@ editTable (void) {
 #if defined(USE_CURSES)
     endwin();
 #else /* standard input/output */
+    if (inputAttributesChanged) {
+#if defined(__MINGW32__)
+      SetConsoleMode(STDIN_HANDLE, inputConsoleMode);
+#else /* termios */
+      tcsetattr(STDIN_FILENO, TCSAFLUSH, &inputTerminalAttributes);
+#endif /* input terminal restoration */
+    }
 #endif /* restore keyboard and screen */
 
     if (haveBrailleDisplay(&etd)) releaseBrailleDisplay(&etd);
