@@ -2,13 +2,14 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2008 by The BRLTTY Developers.
+ * Copyright (C) 1995-2010 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
  * This is free software, placed under the terms of the
  * GNU General Public License, as published by the Free Software
- * Foundation.  Please see the file COPYING for details.
+ * Foundation; either version 2 of the License, or (at your option) any
+ * later version. Please see the file LICENSE-GPL for details.
  *
  * Web Page: http://mielke.cc/brltty/
  *
@@ -77,6 +78,8 @@ extern void playTune (const TuneDefinition *tune);
 extern const char *const midiInstrumentTable[];
 extern const unsigned int midiInstrumentCount;
 
+extern int showDotPattern (unsigned char dots, unsigned char duration);
+
 #ifdef __cplusplus
 }
 #endif /* __cplusplus */
