@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2009 by The BRLTTY Developers.
+ * Copyright (C) 1995-2010 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -37,14 +37,25 @@
 #define USBDEVFS_CONNECT _IO('U', 23)
 #endif /* USBDEVFS_CONNECT */
 
+#include "log.h"
+#include "file.h"
+#include "parse.h"
+#include "timing.h"
 #include "misc.h"
 #include "mount.h"
 #include "io_usb.h"
 #include "usb_internal.h"
 
-struct UsbDeviceExtensionStruct {
-  char *usbfsPath;
+typedef struct {
   char *sysfsPath;
+  char *usbfsPath;
+  UsbDeviceDescriptor usbDescriptor;
+} UsbHostDevice;
+
+static Queue *usbHostDevices = NULL;
+
+struct UsbDeviceExtensionStruct {
+  const UsbHostDevice *host;
   int usbfsFile;
 };
 
@@ -55,9 +66,9 @@ struct UsbEndpointExtensionStruct {
 static int
 usbOpenUsbfsFile (UsbDeviceExtension *devx) {
   if (devx->usbfsFile == -1) {
-    if ((devx->usbfsFile = open(devx->usbfsPath, O_RDWR)) == -1) {
+    if ((devx->usbfsFile = open(devx->host->usbfsPath, O_RDWR)) == -1) {
       LogPrint(LOG_ERR, "USBFS open error: %s: %s",
-               devx->usbfsPath, strerror(errno));
+               devx->host->usbfsPath, strerror(errno));
       return 0;
     }
   }
@@ -90,8 +101,8 @@ usbDisableAutosuspend (UsbDevice *device) {
   UsbDeviceExtension *devx = device->extension;
   int ok = 0;
 
-  if (devx->sysfsPath) {
-    char *path = makePath(devx->sysfsPath, "power/autosuspend");
+  if (devx->host->sysfsPath) {
+    char *path = makePath(devx->host->sysfsPath, "power/autosuspend");
 
     if (path) {
       int file = open(path, O_WRONLY);
@@ -694,53 +705,8 @@ usbWriteEndpoint (
 
 int
 usbReadDeviceDescriptor (UsbDevice *device) {
-  UsbDeviceExtension *devx = device->extension;
-  int file = -1;
-  int sysfs = 0;
-
-  if (devx->sysfsPath) {
-    if (file == -1) {
-      char *path;
-
-      if ((path = makePath(devx->sysfsPath, "descriptors"))) {
-        if ((file = open(path, O_RDONLY)) != -1) {
-          sysfs = 1;
-        }
-
-        free(path);
-      }
-    }
-  }
-
-  if (file == -1) {
-    if (usbOpenUsbfsFile(devx)) {
-      file = devx->usbfsFile;
-    }
-  }
-
-  if (file != -1) {
-    int count = read(file, &device->descriptor, UsbDescriptorSize_Device);
-
-    if (count == -1) {
-      LogError("USB device descriptor read");
-    } else if (count != UsbDescriptorSize_Device) {
-      LogPrint(LOG_ERR, "USB short device descriptor: %d", count);
-    } else {
-      if (sysfs) {
-        device->descriptor.bcdUSB = getLittleEndian(device->descriptor.bcdUSB);
-        device->descriptor.idVendor = getLittleEndian(device->descriptor.idVendor);
-        device->descriptor.idProduct = getLittleEndian(device->descriptor.idProduct);
-        device->descriptor.bcdDevice = getLittleEndian(device->descriptor.bcdDevice);
-
-        close(file);
-        file = -1;
-      }
-
-      return 1;
-    }
-  }
-
-  return 0;
+  device->descriptor = device->extension->host->usbDescriptor;
+  return 1;
 }
 
 int
@@ -754,6 +720,8 @@ usbAllocateEndpointExtension (UsbEndpoint *endpoint) {
     } else {
       LogError("USB endpoint completed request queue allocate");
     }
+
+    free(eptx);
   } else {
     LogError("USB endpoint extension allocate");
   }
@@ -774,28 +742,53 @@ usbDeallocateEndpointExtension (UsbEndpointExtension *eptx) {
 void
 usbDeallocateDeviceExtension (UsbDeviceExtension *devx) {
   usbCloseUsbfsFile(devx);
-
-  if (devx->usbfsPath) {
-    free(devx->usbfsPath);
-    devx->usbfsPath = NULL;
-  }
-
-  if (devx->sysfsPath) {
-    free(devx->sysfsPath);
-    devx->sysfsPath = NULL;
-  }
-
   free(devx);
 }
 
+static void
+usbDeallocateHostDevice (void *item, void *data) {
+  UsbHostDevice *host = item;
+
+  if (host->sysfsPath) free(host->sysfsPath);
+  if (host->usbfsPath) free(host->usbfsPath);
+  free(host);
+}
+
+typedef struct {
+  UsbDeviceChooser chooser;
+  void *data;
+  UsbDevice *device;
+} UsbTestHostDeviceData;
+
 static int
-usbMakeSysfsPath (UsbDeviceExtension *devx) {
-  const char *tail = devx->usbfsPath + strlen(devx->usbfsPath);
+usbTestHostDevice (void *item, void *data) {
+  const UsbHostDevice *host = item;
+  UsbTestHostDeviceData *test = data;
+  UsbDeviceExtension *devx;
+
+  if ((devx = malloc(sizeof(*devx)))) {
+    memset(devx, 0, sizeof(*devx));
+    devx->host = host;
+    devx->usbfsFile = -1;
+
+    if ((test->device = usbTestDevice(devx, test->chooser, test->data))) return 1;
+
+    usbDeallocateDeviceExtension(devx);
+  } else {
+    LogError("malloc");
+  }
+
+  return 0;
+}
+
+static char *
+usbMakeSysfsPath (const char *usbfsPath) {
+  const char *tail = usbfsPath + strlen(usbfsPath);
 
   {
     int count = 0;
     while (1) {
-      if (tail == devx->usbfsPath) return 0;
+      if (tail == usbfsPath) return 0;
       if (!isPathDelimiter(*--tail)) continue;
       if (++count == 2) break;
     }
@@ -808,18 +801,27 @@ usbMakeSysfsPath (UsbDeviceExtension *devx) {
     int count = sscanf(tail, "/%u/%u%c", &bus, &device, &extra);
 
     if (count == 2) {
+      unsigned int minor = ((bus - 1) << 7) | (device - 1);
+
       static const char *const formats[] = {
-        "/sys/class/usb_device/usbdev%u.%u/device",
-        "/sys/class/usb_endpoint/usbdev%u.%u_ep00/device",
+        "/sys/dev/char/189:%4$u%1$n%2$u%3$u",
+        "/sys/class/usb_device/usbdev%2$u.%3$u/device%1$n",
+        "/sys/class/usb_endpoint/usbdev%2$u.%3$u_ep00/device%1$n",
         NULL
       };
       const char *const *format = formats;
 
       while (*format) {
+        int length;
         char path[strlen(*format) + (2 * 0X10) + 1];
-        snprintf(path, sizeof(path), *format, bus, device);
+
+        snprintf(path, sizeof(path), *format, &length, bus, device, minor);
+        path[length] = 0;
+
         if (access(path, F_OK) != -1) {
-          return (devx->sysfsPath = strdup(path)) != NULL;
+          char *sysfsPath = strdup(path);
+          if (!sysfsPath) LogError("strdup");
+          return sysfsPath;
         }
 
         format += 1;
@@ -827,18 +829,96 @@ usbMakeSysfsPath (UsbDeviceExtension *devx) {
     }
   }
 
-  return 0;
+  return NULL;
 }
 
-static UsbDevice *
-usbSearchUsbfs (const char *root, UsbDeviceChooser chooser, void *data) {
+static int
+usbReadHostDeviceDescriptor (UsbHostDevice *host) {
+  int ok = 0;
+  int file = -1;
+  int sysfs = 0;
+
+  if (file == -1) {
+    if (host->sysfsPath) {
+      char *path;
+
+      if ((path = makePath(host->sysfsPath, "descriptors"))) {
+        if ((file = open(path, O_RDONLY)) != -1) {
+          sysfs = 1;
+        }
+
+        free(path);
+      }
+    }
+  }
+
+  if (file == -1) {
+    file = open(host->usbfsPath, O_RDONLY);
+  }
+
+  if (file != -1) {
+    int count = read(file, &host->usbDescriptor, UsbDescriptorSize_Device);
+
+    if (count == -1) {
+      LogError("USB device descriptor read");
+    } else if (count != UsbDescriptorSize_Device) {
+      LogPrint(LOG_ERR, "USB short device descriptor: %d", count);
+    } else {
+      ok = 1;
+
+      if (sysfs) {
+        host->usbDescriptor.bcdUSB = getLittleEndian(host->usbDescriptor.bcdUSB);
+        host->usbDescriptor.idVendor = getLittleEndian(host->usbDescriptor.idVendor);
+        host->usbDescriptor.idProduct = getLittleEndian(host->usbDescriptor.idProduct);
+        host->usbDescriptor.bcdDevice = getLittleEndian(host->usbDescriptor.bcdDevice);
+      }
+    }
+
+    close(file);
+  }
+
+  return ok;
+}
+
+static int
+usbAddHostDevice (const char *path) {
+  int ok = 0;
+  UsbHostDevice *host;
+
+  if ((host = malloc(sizeof(*host)))) {
+    if ((host->usbfsPath = strdup(path))) {
+      host->sysfsPath = usbMakeSysfsPath(host->usbfsPath);
+
+      if (!usbReadHostDeviceDescriptor(host)) {
+        ok = 1;
+      } else if (enqueueItem(usbHostDevices, host)) {
+        return 1;
+      }
+
+      if (host->sysfsPath) free(host->sysfsPath);
+      free(host->usbfsPath);
+    } else {
+      LogError("strdup");
+    }
+
+    free(host);
+  } else {
+    LogError("malloc");
+  }
+
+  return ok;
+}
+
+static int
+usbAddHostDevices (const char *root) {
+  int ok = 0;
   size_t rootLength = strlen(root);
-  UsbDevice *device = NULL;
   DIR *directory;
 
   if ((directory = opendir(root))) {
     struct dirent *entry;
 
+    ok = 1;
     while ((entry = readdir(directory))) {
       size_t nameLength = strlen(entry->d_name);
       struct stat status;
@@ -849,30 +929,18 @@ usbSearchUsbfs (const char *root, UsbDeviceChooser chooser, void *data) {
       if (stat(path, &status) == -1) continue;
 
       if (S_ISDIR(status.st_mode)) {
-        if ((device = usbSearchUsbfs(path, chooser, data))) break;
+        if (!usbAddHostDevices(path)) ok = 0;
       } else if (S_ISREG(status.st_mode) || S_ISCHR(status.st_mode)) {
-        UsbDeviceExtension *devx;
-
-        if ((devx = malloc(sizeof(*devx)))) {
-          devx->usbfsPath = NULL;
-          devx->usbfsFile = -1;
-          devx->sysfsPath = NULL;
-
-          if ((devx->usbfsPath = strdup(path))) {
-            usbMakeSysfsPath(devx);
-
-            if ((device = usbTestDevice(devx, chooser, data))) break;
-          }
-
-          usbDeallocateDeviceExtension(devx);
-        }
+        if (!usbAddHostDevice(path)) ok = 0;
       }
+
+      if (!ok) break;
     }
 
     closedir(directory);
   }
 
-  return device;
+  return ok;
 }
 
 typedef int (*FileSystemVerifier) (const char *path);
@@ -912,13 +980,12 @@ usbGetFileSystem (const char *type, const FileSystemCandidate *candidates, Mount
     char *directory = makeWritablePath(type);
 
     if (directory) {
-      if (makeDirectory(directory)) {
+      if (ensureDirectory(directory)) {
         if (verify(directory)) return directory;
 
         {
-          const char *components[] = {PACKAGE_NAME, "-", type};
-          int count = ARRAY_COUNT(components);
-          char *name = joinStrings(components, count);
+          const char *strings[] = {PACKAGE_NAME, "-", type};
+          char *name = joinStrings(strings, ARRAY_COUNT(strings));
           if (makeMountPoint(directory, name, type)) return directory;
         }
       }
@@ -965,16 +1032,45 @@ usbGetUsbfs (void) {
 
 UsbDevice *
 usbFindDevice (UsbDeviceChooser chooser, void *data) {
-  UsbDevice *device = NULL;
-  char *root;
+  if (!usbHostDevices) {
+    int ok = 0;
 
-  if ((root = usbGetUsbfs())) {
-    LogPrint(LOG_DEBUG, "USBFS Root: %s", root);
-    device = usbSearchUsbfs(root, chooser, data);
-    free(root);
-  } else {
-    LogPrint(LOG_DEBUG, "USBFS not mounted");
+    if ((usbHostDevices = newQueue(usbDeallocateHostDevice, NULL))) {
+      char *root;
+
+      if ((root = usbGetUsbfs())) {
+        LogPrint(LOG_DEBUG, "USBFS Root: %s", root);
+        if (usbAddHostDevices(root)) ok = 1;
+
+        free(root);
+      } else {
+        LogPrint(LOG_DEBUG, "USBFS not mounted");
+      }
+
+      if (!ok) {
+        deallocateQueue(usbHostDevices);
+        usbHostDevices = NULL;
+      }
+    }
   }
 
-  return device;
+  if (usbHostDevices) {
+    UsbTestHostDeviceData test = {
+      .chooser = chooser,
+      .data = data,
+      .device = NULL
+    };
+
+    if (processQueue(usbHostDevices, usbTestHostDevice, &test)) return test.device;
+  }
+
+  return NULL;
+}
+
+void
+usbForgetDevices (void) {
+  if (usbHostDevices) {
+    deallocateQueue(usbHostDevices);
+    usbHostDevices = NULL;
+  }
 }
