@@ -2,13 +2,14 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2008 by The BRLTTY Developers.
+ * Copyright (C) 1995-2010 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
  * This is free software, placed under the terms of the
  * GNU General Public License, as published by the Free Software
- * Foundation.  Please see the file COPYING for details.
+ * Foundation; either version 2 of the License, or (at your option) any
+ * later version. Please see the file LICENSE-GPL for details.
  *
  * Web Page: http://mielke.cc/brltty/
  *
@@ -22,7 +23,7 @@
 extern "C" {
 #endif /* __cplusplus */
 
-#define PACKAGE_COPYRIGHT "Copyright (C) 1995-2008 by The BRLTTY Developers."
+#define PACKAGE_COPYRIGHT "Copyright (C) 1995-2010 by The BRLTTY Developers."
 #define PACKAGE_URL "http://mielke.cc/brltty/"
 
 #define _CONCATENATE(a,b) a##b
@@ -31,6 +32,9 @@ extern "C" {
 #define _STRINGIFY(a) #a
 #define STRINGIFY(a) _STRINGIFY(a)
 
+#define MIN(a, b)  (((a) < (b))? (a): (b)) 
+#define MAX(a, b)  (((a) > (b))? (a): (b)) 
+
 #define ARRAY_COUNT(array) (sizeof((array)) / sizeof((array)[0]))
 #define ARRAY_SIZE(pointer, count) ((count) * sizeof(*(pointer)))
 
@@ -41,20 +45,41 @@ extern "C" {
 #define WINVER WindowsXP
 
 #ifdef __MINGW32__
+#ifndef __USE_W32_SOCKETS
 #define __USE_W32_SOCKETS
+#endif /* __USE_W32_SOCKETS */
 #endif /* __MINGW32__ */
-#include <windows.h>
 
+#include <windows.h>
 #endif /* WINDOWS */
 
 #ifdef __MINGW32__
 #include <_mingw.h>
 #endif /* __MINGW32__ */
 
+/*
+ * The (poorly named) macro "interface" is unfortunately defined within
+ * Windows headers. Fortunately, though, it's also only ever used within
+ * them so it's safe to undefine it here.
+ */
+#ifdef interface
+#undef interface
+#endif /* interface */
+
 #include <sys/types.h>
 #include <stdlib.h>
 #include <unistd.h>
 
+#ifdef __MINGW32__
+#if (__MINGW32_MAJOR_VERSION < 3) || ((__MINGW32_MAJOR_VERSION == 3) && (__MINGW32_MINOR_VERSION < 10))
+extern int gettimeofday (struct timeval *tvp, void *tzp);
+#endif /* gettimeofday */
+
+#if (__MINGW32_MAJOR_VERSION < 3) || ((__MINGW32_MAJOR_VERSION == 3) && (__MINGW32_MINOR_VERSION < 15))
+extern void usleep (int usec);
+#endif /* usleep */
+#endif /* __MINGW32__ */
+
 #ifdef __MSDOS__
 #include <stdarg.h>
 
@@ -74,17 +99,22 @@ typedef signed long long	int64_t;
 
 #define INT8_C(c)	c
 #define INT16_C(c)	c
-#define INT32_C(c)	c
+#define INT32_C(c)	c ## L
 #define INT64_C(c)	c ## LL
 
 #define UINT8_C(c)	c ## U
 #define UINT16_C(c)	c ## U
-#define UINT32_C(c)	c ## U
+#define UINT32_C(c)	c ## UL
 #define UINT64_C(c)	c ## ULL
 
 #define INTMAX_C(c)	c ## LL
 #define UINTMAX_C(c)	c ## ULL
 
+#define PRIu32 "lu"
+#define PRIX32 "lX"
+#define PRIu16 "u"
+#define PRIX16 "X"
+
 #else /* __MSDOS__ */
 #include <inttypes.h>
 #endif /* __MSDOS__ */
@@ -92,25 +122,54 @@ typedef signed long long	int64_t;
 #ifdef __MINGW32__
 typedef HANDLE FileDescriptor;
 #define INVALID_FILE_DESCRIPTOR INVALID_HANDLE_VALUE
-#define PRIFD "p"
+#define PRIfd "p"
 #define closeFileDescriptor(fd) CloseHandle(fd)
 
 typedef SOCKET SocketDescriptor;
 #define INVALID_SOCKET_DESCRIPTOR -1
-#define PRISD "d"
+#define PRIsd "d"
 #define closeSocketDescriptor(sd) closesocket(sd)
 #else /* __MINGW32__ */
 typedef int FileDescriptor;
 #define INVALID_FILE_DESCRIPTOR -1
-#define PRIFD "d"
+#define PRIfd "d"
 #define closeFileDescriptor(fd) close(fd)
 
 typedef int SocketDescriptor;
 #define INVALID_SOCKET_DESCRIPTOR -1
-#define PRISD "d"
+#define PRIsd "d"
 #define closeSocketDescriptor(sd) close(sd)
 #endif /* __MINGW32__ */
 
+#ifdef WINDOWS
+#define getSystemError() GetLastError()
+
+#ifdef __CYGWIN32__
+#include <sys/cygwin.h>
+
+#define getSocketError() errno
+#define setErrno(error) errno = cygwin_internal(CW_GET_ERRNO_FROM_WINERROR, (error))
+#else /* __CYGWIN32__ */
+#define getSocketError() WSAGetLastError()
+#define setErrno(error) errno = win_toErrno((error))
+
+#ifndef WIN_ERRNO_STORAGE_CLASS
+#define WIN_ERRNO_STORAGE_CLASS
+extern
+#endif /* WIN_ERRNO_STORAGE_CLASS */
+WIN_ERRNO_STORAGE_CLASS int win_toErrno (DWORD error);
+#endif /* __CYGWIN32__ */
+
+#else /* WINDOWS */
+#define getSystemError() errno
+#define getSocketError() errno
+
+#define setErrno(error)
+#endif /* WINDOWS */
+
+#define setSystemErrno() setErrno(getSystemError())
+#define setSocketErrno() setErrno(getSocketError())
+
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif /* HAVE_CONFIG_H */
@@ -161,6 +220,9 @@ typedef int SocketDescriptor;
 #define wcswcs strstr
 #define wcsxfrm strxfrm
 
+#define wcstol strtol
+#define wcstoll strtoll
+
 #define iswalnum isalnum
 #define iswalpha isalpha
 #define iswblank isblank
@@ -200,7 +262,7 @@ typedef int SocketDescriptor;
 #endif /* PACKED */
 
 #ifdef HAVE_ATTRIBUTE_FORMAT_PRINTF
-#define PRINTF(fmt,var) __attribute__((format(printf, fmt, var)))
+#define PRINTF(fmt,var) __attribute__((format(__printf__, fmt, var)))
 #else /* HAVE_ATTRIBUTE_FORMAT_PRINTF */
 #define PRINTF(fmt,var)
 #endif /* HAVE_ATTRIBUTE_FORMAT_PRINTF */
